@@ -2,6 +2,7 @@ from __future__ import annotations
 
 from asyncio import Event, Queue, run
 from collections import deque
+from collections.abc import ItemsView, KeysView, ValuesView
 from contextlib import asynccontextmanager
 from dataclasses import dataclass, field
 from itertools import chain
@@ -13,13 +14,14 @@ from hypothesis.strategies import (
     DataObject,
     booleans,
     data,
+    dictionaries,
     integers,
     lists,
     none,
     permutations,
     sampled_from,
 )
-from pytest import LogCaptureFixture, approx, mark, param, raises
+from pytest import LogCaptureFixture, RaisesGroup, approx, mark, param, raises
 
 from tests.conftest import IS_CI
 from tests.test_asyncio_classes.loopers import (
@@ -38,6 +40,7 @@ from tests.test_asyncio_classes.loopers import (
     assert_looper_stats,
 )
 from utilities.asyncio import (
+    AsyncDict,
     EnhancedQueue,
     EnhancedTaskGroup,
     Looper,
@@ -58,12 +61,12 @@ from utilities.asyncio import (
 )
 from utilities.dataclasses import replace_non_sentinel
 from utilities.functions import get_class_name
-from utilities.hypothesis import sentinels, text_ascii
+from utilities.hypothesis import pairs, sentinels, text_ascii
 from utilities.iterables import one, unique_everseen
 from utilities.pytest import skipif_windows
 from utilities.sentinel import Sentinel, sentinel
 from utilities.timer import Timer
-from utilities.whenever import SECOND, get_now
+from utilities.whenever import MILLISECOND, SECOND, get_now
 
 if TYPE_CHECKING:
     from collections.abc import AsyncIterator
@@ -72,6 +75,155 @@ if TYPE_CHECKING:
 
     from utilities.types import MaybeCallableEvent
 
+async_dicts = dictionaries(text_ascii(), integers()).map(AsyncDict)
+
+
+class TestAsyncDict:
+    @given(dict_=async_dicts)
+    async def test_aenter(self, *, dict_: AsyncDict[str, int]) -> None:
+        async with dict_:
+            ...
+
+    @given(dict_=async_dicts)
+    async def test_clear(self, *, dict_: AsyncDict[str, int]) -> None:
+        await dict_.clear()
+        assert len(dict_) == 0
+
+    @given(dict_=async_dicts, key=text_ascii())
+    def test_contains(self, *, dict_: AsyncDict[str, int], key: str) -> None:
+        assert isinstance(key in dict_, bool)
+
+    @given(dict_=async_dicts)
+    def test_copy(self, *, dict_: AsyncDict[str, int]) -> None:
+        assert isinstance(dict_.copy(), AsyncDict)
+
+    @given(dict_=async_dicts, key=text_ascii())
+    async def test_del(self, *, dict_: AsyncDict[str, int], key: str) -> None:
+        if key in dict_:
+            await dict_.del_(key)
+        else:
+            with raises(KeyError):
+                await dict_.del_(key)
+
+    @given(dict_=async_dicts)
+    def test_empty(self, *, dict_: AsyncDict[str, int]) -> None:
+        assert isinstance(dict_.empty, bool)
+
+    @given(dicts=pairs(async_dicts))
+    def test_eq(
+        self, *, dicts: tuple[AsyncDict[str, int], AsyncDict[str, int]]
+    ) -> None:
+        first, second = dicts
+        assert isinstance(first == second, bool)
+
+    @given(keys=lists(text_ascii()))
+    def test_fromkeys(self, *, keys: list[str]) -> None:
+        dict_ = AsyncDict.fromkeys(keys)
+        assert isinstance(dict_, AsyncDict)
+
+    @given(dict_=async_dicts, key=text_ascii())
+    def test_get(self, *, dict_: AsyncDict[str, int], key: str) -> None:
+        if key in dict_:
+            assert isinstance(dict_.get(key), int)
+        else:
+            assert dict_.get(key) is None
+
+    @given(dict_=async_dicts, key=text_ascii())
+    def test_get_default(self, *, dict_: AsyncDict[str, int], key: str) -> None:
+        value = dict_.get(key, None)
+        assert isinstance(value, int) or (value is None)
+
+    @given(dict_=async_dicts, key=text_ascii())
+    def test_getitem(self, *, dict_: AsyncDict[str, int], key: str) -> None:
+        if key in dict_:
+            assert isinstance(dict_[key], int)
+        else:
+            with raises(KeyError):
+                _ = dict_[key]
+
+    @given(dict_=async_dicts)
+    def test_items(self, *, dict_: AsyncDict[str, int]) -> None:
+        assert isinstance(dict_.items(), ItemsView)
+        for key, value in dict_.items():
+            assert isinstance(key, str)
+            assert isinstance(value, int)
+
+    @given(dict_=async_dicts)
+    def test_iter(self, *, dict_: AsyncDict[str, int]) -> None:
+        for key in dict_:
+            assert isinstance(key, str)
+
+    @given(dict_=async_dicts)
+    def test_keys(self, *, dict_: AsyncDict[str, int]) -> None:
+        assert isinstance(dict_.keys(), KeysView)
+        for key in dict_.keys():  # noqa: SIM118
+            assert isinstance(key, str)
+
+    @given(dict_=async_dicts)
+    def test_len(self, *, dict_: AsyncDict[str, int]) -> None:
+        assert isinstance(len(dict_), int)
+
+    @given(dict_=async_dicts, key=text_ascii())
+    async def test_pop(self, *, dict_: AsyncDict[str, int], key: str) -> None:
+        if key in dict_:
+            assert isinstance(await dict_.pop(key), int)
+        else:
+            with raises(KeyError):
+                _ = await dict_.pop(key)
+
+    @given(dict_=async_dicts, key=text_ascii())
+    async def test_pop_default(self, *, dict_: AsyncDict[str, int], key: str) -> None:
+        value = await dict_.pop(key, None)
+        assert isinstance(value, int) or (value is None)
+
+    @given(dict_=async_dicts)
+    async def test_popitem(self, *, dict_: AsyncDict[str, int]) -> None:
+        if len(dict_) >= 1:
+            key, value = await dict_.popitem()
+            assert isinstance(key, str)
+            assert isinstance(value, int)
+        else:
+            with raises(KeyError):
+                _ = await dict_.popitem()
+
+    @given(dict_=async_dicts)
+    def test_repr(self, *, dict_: AsyncDict[str, int]) -> None:
+        assert isinstance(repr(dict_), str)
+
+    @given(dict_=async_dicts)
+    def test_reversed(self, *, dict_: AsyncDict[str, int]) -> None:
+        for key in reversed(dict_):
+            assert isinstance(key, str)
+
+    @given(dict_=async_dicts, key=text_ascii(), value=integers())
+    async def test_set(
+        self, *, dict_: AsyncDict[str, int], key: str, value: int
+    ) -> None:
+        await dict_.set(key, value)
+
+    @given(dict_=async_dicts, key=text_ascii(), value=integers())
+    async def test_setdefault(
+        self, *, dict_: AsyncDict[str, int], key: str, value: int
+    ) -> None:
+        assert isinstance(await dict_.setdefault(key, value), int)
+
+    @given(dict_=async_dicts)
+    def test_str(self, *, dict_: AsyncDict[str, int]) -> None:
+        assert isinstance(str(dict_), str)
+
+    @given(dicts=pairs(async_dicts))
+    async def test_update(
+        self, *, dicts: tuple[AsyncDict[str, int], AsyncDict[str, int]]
+    ) -> None:
+        first, second = dicts
+        await first.update(second)
+
+    @given(dict_=async_dicts)
+    def test_values(self, *, dict_: AsyncDict[str, int]) -> None:
+        assert isinstance(dict_.values(), ValuesView)
+        for value in dict_.values():
+            assert isinstance(value, int)
+
 
 class TestEnhancedQueue:
     @given(
@@ -189,6 +341,22 @@ class TestEnhancedTaskGroup:
                     _ = tg.create_task(sleep_td(self.delta))
         assert timer >= 5 * self.delta
 
+    async def test_run_or_create_many_tasks_parallel_with_max_tasks_two(self) -> None:
+        with Timer() as timer:
+            async with EnhancedTaskGroup(max_tasks=2) as tg:
+                assert not tg._is_debug()
+                for _ in range(10):
+                    _ = await tg.run_or_create_many_tasks(sleep_td, self.delta)
+        assert timer >= 5 * self.delta
+
+    async def test_run_or_create_many_tasks_serial_with_debug(self) -> None:
+        with Timer() as timer:
+            async with EnhancedTaskGroup(debug=True) as tg:
+                assert tg._is_debug()
+                for _ in range(10):
+                    _ = await tg.run_or_create_many_tasks(sleep_td, self.delta)
+        assert timer >= 10 * self.delta
+
     async def test_run_or_create_task_parallel_with_max_tasks_none(self) -> None:
         with Timer() as timer:
             async with EnhancedTaskGroup() as tg:
@@ -226,22 +394,16 @@ class TestEnhancedTaskGroup:
             _ = tg.create_task(sleep_td(self.delta))
 
     async def test_timeout_fail(self) -> None:
-        with raises(ExceptionGroup) as exc_info:
+        with RaisesGroup(TimeoutError):
             async with EnhancedTaskGroup(timeout=self.delta) as tg:
                 _ = tg.create_task(sleep_td(2 * self.delta))
-        assert len(exc_info.value.exceptions) == 1
-        error = one(exc_info.value.exceptions)
-        assert isinstance(error, TimeoutError)
 
     async def test_custom_error(self) -> None:
         class CustomError(Exception): ...
 
-        with raises(ExceptionGroup) as exc_info:
+        with RaisesGroup(CustomError):
             async with EnhancedTaskGroup(timeout=self.delta, error=CustomError) as tg:
                 _ = tg.create_task(sleep_td(2 * self.delta))
-        assert len(exc_info.value.exceptions) == 1
-        error = one(exc_info.value.exceptions)
-        assert isinstance(error, CustomError)
 
 
 class TestGetEvent:
@@ -421,11 +583,9 @@ class TestLooper:
         looper = LooperWithCounterMixin(
             auto_start=True, timeout=SECOND, counter_auto_start=counter_auto_start
         )
-        with raises(ExceptionGroup) as exc_info:
+        with RaisesGroup(TimeoutError):
             async with EnhancedTaskGroup(timeout=looper.timeout) as tg:
                 _ = tg.create_task_context(looper)
-        error = one(exc_info.value.exceptions)
-        assert isinstance(error, TimeoutError)
         self._assert_stats_half(looper._counter)
 
     @mark.parametrize("counter1_auto_start", [param(True), param(False)])
@@ -469,14 +629,11 @@ class TestLooper:
         looper2 = LooperWithCounterMixin(
             auto_start=True, timeout=SECOND, counter_auto_start=counter_auto_start
         )
-        with raises(ExceptionGroup) as exc_info:  # noqa: PT012
+        with raises(ExceptionGroup) as exc_info:
             async with EnhancedTaskGroup(timeout=SECOND) as tg:
                 _ = tg.create_task_context(looper1)
                 _ = tg.create_task_context(looper2)
-        errors = exc_info.value.exceptions
-        assert 1 <= len(errors) <= 2
-        for error in errors:
-            assert isinstance(error, TimeoutError)
+        assert exc_info.group_contains(TimeoutError)
         self._assert_stats_half(looper1._counter)
         self._assert_stats_half(looper2._counter)
 
@@ -971,7 +1128,7 @@ class TestSleepUntil:
 
 class TestSleepUntilRounded:
     async def test_main(self) -> None:
-        await sleep_rounded(unit="millisecond", increment=10)
+        await sleep_rounded(10 * MILLISECOND)
 
 
 class TestStreamCommand:
