@@ -2,31 +2,19 @@ from __future__ import annotations
 
 from dataclasses import dataclass
 from functools import wraps
-from typing import TYPE_CHECKING, Any, Generic, TypeVar, cast
+from typing import TYPE_CHECKING, Any, cast
 
 from memory_profiler import memory_usage
-from typing_extensions import ParamSpec
 
 if TYPE_CHECKING:
     from collections.abc import Callable
 
-_P = ParamSpec("_P")
-_T = TypeVar("_T")
 
-
-@dataclass(kw_only=True, slots=True)
-class Output(Generic[_T]):
-    """A function output, and its memory usage."""
-
-    value: _T
-    memory: float
-
-
-def memory_profiled(func: Callable[_P, _T], /) -> Callable[_P, Output[_T]]:
+def memory_profiled[**P, T](func: Callable[P, T], /) -> Callable[P, Output[T]]:
     """Call a function, but also profile its maximum memory usage."""
 
     @wraps(func)
-    def wrapped(*args: _P.args, **kwargs: _P.kwargs) -> Output[_T]:
+    def wrapped(*args: P.args, **kwargs: P.kwargs) -> Output[T]:
         memory, value = memory_usage(
             cast("Any", (func, args, kwargs)), max_usage=True, retval=True
         )
@@ -35,4 +23,12 @@ def memory_profiled(func: Callable[_P, _T], /) -> Callable[_P, Output[_T]]:
     return wrapped
 
 
+@dataclass(kw_only=True, slots=True)
+class Output[T]:
+    """A function output, and its memory usage."""
+
+    value: T
+    memory: float
+
+
 __all__ = ["Output", "memory_profiled"]
