@@ -14,6 +14,7 @@ from pathlib import Path
 from re import Pattern, search
 from typing import TYPE_CHECKING, Any, Literal, Self, assert_never, overload, override
 from uuid import UUID
+from zoneinfo import ZoneInfo
 
 from orjson import (
     OPT_PASSTHROUGH_DATACLASS,
@@ -22,10 +23,22 @@ from orjson import (
     dumps,
     loads,
 )
+from whenever import (
+    Date,
+    DateDelta,
+    DateTimeDelta,
+    MonthDay,
+    PlainDateTime,
+    Time,
+    TimeDelta,
+    YearMonth,
+    ZonedDateTime,
+)
 
 from utilities.concurrent import concurrent_map
 from utilities.dataclasses import dataclass_to_dict
 from utilities.functions import ensure_class, is_string_mapping
+from utilities.gzip import read_binary
 from utilities.iterables import (
     OneEmptyError,
     always_iterable,
@@ -33,32 +46,13 @@ from utilities.iterables import (
     one,
     one_unique,
 )
+from utilities.json import write_formatted_json
 from utilities.logging import get_logging_level_number
 from utilities.math import MAX_INT64, MIN_INT64
-from utilities.types import (
-    Dataclass,
-    DateOrDateTime,
-    LogLevel,
-    MaybeIterable,
-    PathLike,
-    StrMapping,
-)
-from utilities.tzlocal import get_local_time_zone
-from utilities.uuid import UUID_PATTERN
+from utilities.types import Dataclass, LogLevel, MaybeIterable, PathLike, StrMapping
+from utilities.tzlocal import LOCAL_TIME_ZONE
 from utilities.version import Version, parse_version
-from utilities.whenever import (
-    parse_date,
-    parse_local_datetime,
-    parse_time,
-    parse_timedelta,
-    parse_zoned_datetime,
-    serialize_date,
-    serialize_local_datetime,
-    serialize_time,
-    serialize_timedelta,
-    serialize_zoned_datetime,
-)
-from utilities.zoneinfo import ensure_time_zone
+from utilities.whenever import from_timestamp
 
 if TYPE_CHECKING:
     from collections.abc import Set as AbstractSet
@@ -74,25 +68,31 @@ if TYPE_CHECKING:
 class _Prefixes(Enum):
     dataclass = "dc"
     date = "d"
-    datetime = "dt"
+    date_delta = "dd"
+    date_time_delta = "D"
     enum = "e"
-    exception_class = "exc"
-    exception_instance = "exi"
+    exception_class = "Ex"
+    exception_instance = "ex"
     float_ = "fl"
     frozenset_ = "fr"
     list_ = "l"
-    nan = "nan"
+    month_day = "md"
     none = "none"
     path = "p"
-    pos_inf = "pos_inf"
-    neg_inf = "neg_inf"
+    plain_date_time = "pd"
+    py_date = "!d"
+    py_plain_date_time = "!pd"
+    py_time = "!ti"
+    py_zoned_date_time = "!zd"
     set_ = "s"
-    timedelta = "td"
-    time = "tm"
+    time = "ti"
+    time_delta = "td"
     tuple_ = "tu"
     unserializable = "un"
     uuid = "uu"
     version = "v"
+    year_month = "ym"
+    zoned_date_time = "zd"
 
 
 type _DataclassHook = Callable[[type[Dataclass], StrMapping], StrMapping]
@@ -115,6 +115,7 @@ def serialize(
     before: Callable[[Any], Any] | None = None,
     globalns: StrMapping | None = None,
     localns: StrMapping | None = None,
+    warn_name_errors: bool = False,
     dataclass_hook: _DataclassHook | None = None,
     dataclass_defaults: bool = False,
 ) -> bytes:
@@ -124,6 +125,7 @@ def serialize(
         before=before,
         globalns=globalns,
         localns=localns,
+        warn_name_errors=warn_name_errors,
         dataclass_hook=dataclass_hook,
         dataclass_defaults=dataclass_defaults,
     )
@@ -140,6 +142,7 @@ def _pre_process(
     before: Callable[[Any], Any] | None = None,
     globalns: StrMapping | None = None,
     localns: StrMapping | None = None,
+    warn_name_errors: bool = False,
     dataclass_hook: _DataclassHook | None = None,
     dataclass_defaults: bool = False,
     error: _ErrorMode = "raise",
@@ -151,6 +154,7 @@ def _pre_process(
         before=before,
         globalns=globalns,
         localns=localns,
+        warn_name_errors=warn_name_errors,
         dataclass_hook=dataclass_hook,
         dataclass_defaults=dataclass_defaults,
         error=error,
@@ -159,20 +163,12 @@ def _pre_process(
         # singletons
         case None:
             return f"[{_Prefixes.none.value}]"
-        case dt.datetime() as datetime:
-            if datetime.tzinfo is None:
-                ser = serialize_local_datetime(datetime)
-            elif datetime.tzinfo is dt.UTC:
-                ser = serialize_zoned_datetime(datetime).replace("UTC", "dt.UTC")
-            else:
-                ser = serialize_zoned_datetime(datetime)
-            return f"[{_Prefixes.datetime.value}]{ser}"
-        case dt.date() as date:  # after datetime
-            return f"[{_Prefixes.date.value}]{serialize_date(date)}"
-        case dt.time() as time:
-            return f"[{_Prefixes.time.value}]{serialize_time(time)}"
-        case dt.timedelta() as timedelta:
-            return f"[{_Prefixes.timedelta.value}]{serialize_timedelta(timedelta)}"
+        case Date() as date:
+            return f"[{_Prefixes.date.value}]{date}"
+        case DateDelta() as date:
+            return f"[{_Prefixes.date_delta.value}]{date}"
+        case DateTimeDelta() as date:
+            return f"[{_Prefixes.date_time_delta.value}]{date}"
         case Exception() as error_:
             return {
                 f"[{_Prefixes.exception_instance.value}|{type(error_).__qualname__}]": pre(
@@ -187,22 +183,51 @@ def _pre_process(
             if MIN_INT64 <= int_ <= MAX_INT64:
                 return int_
             raise _SerializeIntegerError(obj=int_)
-        case UUID() as uuid:
-            return f"[{_Prefixes.uuid.value}]{uuid}"
+        case MonthDay() as month_day:
+            return f"[{_Prefixes.month_day.value}]{month_day!s}"
         case Path() as path:
             return f"[{_Prefixes.path.value}]{path!s}"
+        case PlainDateTime() as datetime:
+            return f"[{_Prefixes.plain_date_time.value}]{datetime}"
         case str() as str_:
             return str_
+        case Time() as time:
+            return f"[{_Prefixes.time.value}]{time}"
+        case TimeDelta() as time_delta:
+            return f"[{_Prefixes.time_delta.value}]{time_delta}"
         case type() as error_cls if issubclass(error_cls, Exception):
             return f"[{_Prefixes.exception_class.value}|{error_cls.__qualname__}]"
+        case UUID() as uuid:
+            return f"[{_Prefixes.uuid.value}]{uuid}"
         case Version() as version:
-            return f"[{_Prefixes.version.value}]{version!s}"
+            return f"[{_Prefixes.version.value}]{version}"
+        case YearMonth() as year_month:
+            return f"[{_Prefixes.year_month.value}]{year_month}"
+        case ZonedDateTime() as datetime:
+            return f"[{_Prefixes.zoned_date_time.value}]{datetime}"
+        case dt.datetime() as py_datetime:
+            match py_datetime.tzinfo:
+                case None:
+                    datetime = PlainDateTime.from_py_datetime(py_datetime)
+                    return f"[{_Prefixes.py_plain_date_time.value}]{datetime}"
+                case ZoneInfo():
+                    datetime = ZonedDateTime.from_py_datetime(py_datetime)
+                    return f"[{_Prefixes.py_zoned_date_time.value}]{datetime}"
+                case _:  # pragma: no cover
+                    raise NotImplementedError
+        case dt.date() as py_date:
+            date = Date.from_py_date(py_date)
+            return f"[{_Prefixes.py_date.value}]{date}"
+        case dt.time() as py_time:
+            time = Time.from_py_time(py_time)
+            return f"[{_Prefixes.py_time.value}]{time}"
         # contains
         case Dataclass() as dataclass:
             asdict = dataclass_to_dict(
                 dataclass,
                 globalns=globalns,
                 localns=localns,
+                warn_name_errors=warn_name_errors,
                 final=partial(_dataclass_final, hook=dataclass_hook),
                 defaults=dataclass_defaults,
             )
@@ -219,6 +244,7 @@ def _pre_process(
                 before=before,
                 globalns=globalns,
                 localns=localns,
+                warn_name_errors=warn_name_errors,
                 dataclass_hook=dataclass_hook,
             )
         case list() as list_:
@@ -229,6 +255,7 @@ def _pre_process(
                 before=before,
                 globalns=globalns,
                 localns=localns,
+                warn_name_errors=warn_name_errors,
                 dataclass_hook=dataclass_hook,
             )
         case Mapping() as mapping:
@@ -241,6 +268,7 @@ def _pre_process(
                 before=before,
                 globalns=globalns,
                 localns=localns,
+                warn_name_errors=warn_name_errors,
                 dataclass_hook=dataclass_hook,
             )
         case tuple() as tuple_:
@@ -251,6 +279,7 @@ def _pre_process(
                 before=before,
                 globalns=globalns,
                 localns=localns,
+                warn_name_errors=warn_name_errors,
                 dataclass_hook=dataclass_hook,
             )
         # other
@@ -259,6 +288,7 @@ def _pre_process(
                 qualname=type(obj).__qualname__, repr=repr(obj), str=str(obj)
             )
             return pre(unserializable)
+    return None
 
 
 def _pre_process_container(
@@ -270,6 +300,7 @@ def _pre_process_container(
     before: Callable[[Any], Any] | None = None,
     globalns: StrMapping | None = None,
     localns: StrMapping | None = None,
+    warn_name_errors: bool = False,
     dataclass_hook: _DataclassHook | None = None,
     dataclass_include_defaults: bool = False,
 ) -> Any:
@@ -279,6 +310,7 @@ def _pre_process_container(
             before=before,
             globalns=globalns,
             localns=localns,
+            warn_name_errors=warn_name_errors,
             dataclass_hook=dataclass_hook,
             dataclass_defaults=dataclass_include_defaults,
         )
@@ -334,51 +366,48 @@ def deserialize(
     )
 
 
-_NONE_PATTERN = re.compile(r"^\[" + _Prefixes.none.value + r"\]$")
-_LOCAL_DATETIME_PATTERN = re.compile(
-    r"^\["
-    + _Prefixes.datetime.value
-    + r"\](\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}(?:\.\d{1,6})?)$"
-)
-_UUID_PATTERN = re.compile(r"^\[" + _Prefixes.uuid.value + r"\](" + UUID_PATTERN + ")$")
-_ZONED_DATETIME_PATTERN = re.compile(
-    r"^\["
-    + _Prefixes.datetime.value
-    + r"\](\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}(?:\.\d{1,6})?[\+\-]\d{2}:\d{2}(?::\d{2})?\[(?!(?:dt\.)).+?\])$"
-)
-_ZONED_DATETIME_ALTERNATIVE_PATTERN = re.compile(
-    r"^\["
-    + _Prefixes.datetime.value
-    + r"\](\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}(?:\.\d{1,6})?\+00:00\[dt\.UTC\])$"
-)
-
-
-def _make_unit_pattern(prefix: _Prefixes, /) -> Pattern[str]:
-    return re.compile(r"^\[" + prefix.value + r"\](.+)$")
-
-
 (
     _DATE_PATTERN,
+    _DATE_DELTA_PATTERN,
+    _DATE_TIME_DELTA_PATTERN,
     _FLOAT_PATTERN,
+    _MONTH_DAY_PATTERN,
+    _NONE_PATTERN,
     _PATH_PATTERN,
+    _PLAIN_DATE_TIME_PATTERN,
+    _PY_DATE_PATTERN,
+    _PY_PLAIN_DATE_TIME_PATTERN,
+    _PY_TIME_PATTERN,
+    _PY_ZONED_DATE_TIME_PATTERN,
     _TIME_PATTERN,
-    _TIMEDELTA_PATTERN,
+    _TIME_DELTA_PATTERN,
+    _UUID_PATTERN,
     _VERSION_PATTERN,
-) = map(
-    _make_unit_pattern,
-    [
+    _YEAR_MONTH_PATTERN,
+    _ZONED_DATE_TIME_PATTERN,
+) = [
+    re.compile(r"^\[" + p.value + r"\](" + ".*" + ")$")
+    for p in [
         _Prefixes.date,
+        _Prefixes.date_delta,
+        _Prefixes.date_time_delta,
         _Prefixes.float_,
+        _Prefixes.month_day,
+        _Prefixes.none,
         _Prefixes.path,
+        _Prefixes.plain_date_time,
+        _Prefixes.py_date,
+        _Prefixes.py_plain_date_time,
+        _Prefixes.py_time,
+        _Prefixes.py_zoned_date_time,
         _Prefixes.time,
-        _Prefixes.timedelta,
+        _Prefixes.time_delta,
+        _Prefixes.uuid,
         _Prefixes.version,
-    ],
-)
-
-
-def _make_container_pattern(prefix: _Prefixes, /) -> Pattern[str]:
-    return re.compile(r"^\[" + prefix.value + r"(?:\|(.+))?\]$")
+        _Prefixes.year_month,
+        _Prefixes.zoned_date_time,
+    ]
+]
 
 
 (
@@ -390,9 +419,9 @@ def _make_container_pattern(prefix: _Prefixes, /) -> Pattern[str]:
     _LIST_PATTERN,
     _SET_PATTERN,
     _TUPLE_PATTERN,
-) = map(
-    _make_container_pattern,
-    [
+) = [
+    re.compile(r"^\[" + p.value + r"(?:\|(.+))?\]$")
+    for p in [
         _Prefixes.dataclass,
         _Prefixes.enum,
         _Prefixes.exception_class,
@@ -401,8 +430,8 @@ def _make_container_pattern(prefix: _Prefixes, /) -> Pattern[str]:
         _Prefixes.list_,
         _Prefixes.set_,
         _Prefixes.tuple_,
-    ],
-)
+    ]
+]
 
 
 def _object_hook(
@@ -421,27 +450,39 @@ def _object_hook(
             if match := _NONE_PATTERN.search(text):
                 return None
             if match := _DATE_PATTERN.search(text):
-                return parse_date(match.group(1))
+                return Date.parse_common_iso(match.group(1))
+            if match := _DATE_DELTA_PATTERN.search(text):
+                return DateDelta.parse_common_iso(match.group(1))
+            if match := _DATE_TIME_DELTA_PATTERN.search(text):
+                return DateTimeDelta.parse_common_iso(match.group(1))
             if match := _FLOAT_PATTERN.search(text):
                 return float(match.group(1))
-            if match := _LOCAL_DATETIME_PATTERN.search(text):
-                return parse_local_datetime(match.group(1))
+            if match := _MONTH_DAY_PATTERN.search(text):
+                return MonthDay.parse_common_iso(match.group(1))
             if match := _PATH_PATTERN.search(text):
                 return Path(match.group(1))
+            if match := _PLAIN_DATE_TIME_PATTERN.search(text):
+                return PlainDateTime.parse_common_iso(match.group(1))
+            if match := _PY_DATE_PATTERN.search(text):
+                return Date.parse_common_iso(match.group(1)).py_date()
+            if match := _PY_PLAIN_DATE_TIME_PATTERN.search(text):
+                return PlainDateTime.parse_common_iso(match.group(1)).py_datetime()
+            if match := _PY_TIME_PATTERN.search(text):
+                return Time.parse_common_iso(match.group(1)).py_time()
+            if match := _PY_ZONED_DATE_TIME_PATTERN.search(text):
+                return ZonedDateTime.parse_common_iso(match.group(1)).py_datetime()
             if match := _TIME_PATTERN.search(text):
-                return parse_time(match.group(1))
-            if match := _TIMEDELTA_PATTERN.search(text):
-                return parse_timedelta(match.group(1))
+                return Time.parse_common_iso(match.group(1))
+            if match := _TIME_DELTA_PATTERN.search(text):
+                return TimeDelta.parse_common_iso(match.group(1))
             if match := _UUID_PATTERN.search(text):
                 return UUID(match.group(1))
             if match := _VERSION_PATTERN.search(text):
                 return parse_version(match.group(1))
-            if match := _ZONED_DATETIME_PATTERN.search(text):
-                return parse_zoned_datetime(match.group(1))
-            if match := _ZONED_DATETIME_ALTERNATIVE_PATTERN.search(text):
-                return parse_zoned_datetime(
-                    match.group(1).replace("dt.UTC", "UTC")
-                ).replace(tzinfo=dt.UTC)
+            if match := _YEAR_MONTH_PATTERN.search(text):
+                return YearMonth.parse_common_iso(match.group(1))
+            if match := _ZONED_DATE_TIME_PATTERN.search(text):
+                return ZonedDateTime.parse_common_iso(match.group(1))
             if (
                 exc_class := _object_hook_exception_class(
                     text, data=data, objects=objects, redirects=redirects
@@ -697,6 +738,7 @@ _LOG_RECORD_DEFAULT_ATTRS = {
     "exc_text",
     "filename",
     "funcName",
+    "getMessage",
     "levelname",
     "levelno",
     "lineno",
@@ -732,6 +774,7 @@ class OrjsonFormatter(Formatter):
         before: Callable[[Any], Any] | None = None,
         globalns: StrMapping | None = None,
         localns: StrMapping | None = None,
+        warn_name_errors: bool = False,
         dataclass_hook: _DataclassHook | None = None,
         dataclass_defaults: bool = False,
     ) -> None:
@@ -739,13 +782,12 @@ class OrjsonFormatter(Formatter):
         self._before = before
         self._globalns = globalns
         self._localns = localns
+        self._warn_name_errors = warn_name_errors
         self._dataclass_hook = dataclass_hook
         self._dataclass_defaults = dataclass_defaults
 
     @override
     def format(self, record: LogRecord) -> str:
-        from utilities.tzlocal import get_local_time_zone
-
         extra = {
             k: v
             for k, v in record.__dict__.items()
@@ -757,9 +799,7 @@ class OrjsonFormatter(Formatter):
             path_name=Path(record.pathname),
             line_num=record.lineno,
             message=record.getMessage(),
-            datetime=dt.datetime.fromtimestamp(
-                record.created, tz=get_local_time_zone()
-            ),
+            datetime=from_timestamp(record.created, time_zone=LOCAL_TIME_ZONE),
             func_name=record.funcName,
             extra=extra if len(extra) >= 1 else None,
         )
@@ -768,6 +808,7 @@ class OrjsonFormatter(Formatter):
             before=self._before,
             globalns=self._globalns,
             localns=self._localns,
+            warn_name_errors=self._warn_name_errors,
             dataclass_hook=self._dataclass_hook,
             dataclass_defaults=self._dataclass_defaults,
         ).decode()
@@ -849,9 +890,7 @@ class GetLogRecordsOutput:
 
     @cached_property
     def dataframe(self) -> Any:
-        from polars import DataFrame, Object, String, UInt64
-
-        from utilities.polars import zoned_datetime
+        from polars import DataFrame, Datetime, Object, String, UInt64
 
         records = [
             replace(
@@ -862,11 +901,16 @@ class GetLogRecordsOutput:
             for r in self.records
         ]
         if len(records) >= 1:
-            time_zone = one_unique(ensure_time_zone(r.datetime) for r in records)
+            time_zone = one_unique(ZoneInfo(r.datetime.tz) for r in records)
         else:
-            time_zone = get_local_time_zone()
+            time_zone = LOCAL_TIME_ZONE
         return DataFrame(
-            data=[dataclass_to_dict(r, recursive=False) for r in records],
+            data=[
+                dataclass_to_dict(
+                    replace(r, datetime=r.datetime.py_datetime()), recursive=False
+                )
+                for r in records
+            ],
             schema={
                 "index": UInt64,
                 "name": String,
@@ -874,7 +918,7 @@ class GetLogRecordsOutput:
                 "level": UInt64,
                 "path_name": String,
                 "line_num": UInt64,
-                "datetime": zoned_datetime(time_zone=time_zone),
+                "datetime": Datetime(time_zone=time_zone),
                 "func_name": String,
                 "stack_info": String,
                 "extra": Object,
@@ -894,9 +938,12 @@ class GetLogRecordsOutput:
         level: LogLevel | None = None,
         min_level: LogLevel | None = None,
         max_level: LogLevel | None = None,
-        date_or_datetime: DateOrDateTime | None = None,
-        min_date_or_datetime: DateOrDateTime | None = None,
-        max_date_or_datetime: DateOrDateTime | None = None,
+        date: Date | None = None,
+        min_date: Date | None = None,
+        max_date: Date | None = None,
+        datetime: ZonedDateTime | None = None,
+        min_datetime: ZonedDateTime | None = None,
+        max_datetime: ZonedDateTime | None = None,
         func_name: bool | str | None = None,
         extra: bool | MaybeIterable[str] | None = None,
         log_file: bool | PathLike | None = None,
@@ -935,30 +982,18 @@ class GetLogRecordsOutput:
             records = [
                 r for r in records if r.level <= get_logging_level_number(max_level)
             ]
-        if date_or_datetime is not None:
-            match date_or_datetime:
-                case dt.datetime() as datetime:
-                    records = [r for r in records if r.datetime == datetime]
-                case dt.date() as date:
-                    records = [r for r in records if r.date == date]
-                case _ as never:
-                    assert_never(never)
-        if min_date_or_datetime is not None:
-            match min_date_or_datetime:
-                case dt.datetime() as min_datetime:
-                    records = [r for r in records if r.datetime >= min_datetime]
-                case dt.date() as min_date:
-                    records = [r for r in records if r.date >= min_date]
-                case _ as never:
-                    assert_never(never)
-        if max_date_or_datetime is not None:
-            match max_date_or_datetime:
-                case dt.datetime() as max_datetime:
-                    records = [r for r in records if r.datetime <= max_datetime]
-                case dt.date() as max_date:
-                    records = [r for r in records if r.date <= max_date]
-                case _ as never:
-                    assert_never(never)
+        if date is not None:
+            records = [r for r in records if r.date == date]
+        if min_date is not None:
+            records = [r for r in records if r.date >= min_date]
+        if max_date is not None:
+            records = [r for r in records if r.date <= max_date]
+        if datetime is not None:
+            records = [r for r in records if r.datetime == datetime]
+        if min_datetime is not None:
+            records = [r for r in records if r.datetime >= min_datetime]
+        if max_datetime is not None:
+            records = [r for r in records if r.datetime <= max_datetime]
         if func_name is not None:
             match func_name:
                 case bool() as has_func_name:
@@ -1027,7 +1062,7 @@ class GetLogRecordsOutput:
                 r
                 for r in records
                 if (r.log_file_line_num is not None)
-                and (r.log_file_line_num >= max_log_file_line_num)
+                and (r.log_file_line_num <= max_log_file_line_num)
             ]
         return replace(self, records=records)
 
@@ -1061,7 +1096,7 @@ class OrjsonLogRecord:
     level: int
     path_name: Path
     line_num: int
-    datetime: dt.datetime
+    datetime: ZonedDateTime
     func_name: str | None = None
     stack_info: str | None = None
     extra: StrMapping | None = None
@@ -1069,7 +1104,7 @@ class OrjsonLogRecord:
     log_file_line_num: int | None = None
 
     @cached_property
-    def date(self) -> dt.date:
+    def date(self) -> Date:
         return self.datetime.date()
 
 
@@ -1090,8 +1125,7 @@ def _get_log_records_one(
 ) -> _GetLogRecordsOneOutput:
     path = Path(path)
     try:
-        with path.open() as fh:
-            lines = fh.readlines()
+        lines = path.read_text().splitlines()
     except UnicodeDecodeError as error:  # skipif-ci-and-windows
         return _GetLogRecordsOneOutput(path=path, file_ok=False, other_errors=[error])
     num_lines_blank, num_lines_error = 0, 0
@@ -1164,6 +1198,52 @@ class _GetLogRecordsOneOutput:
     other_errors: list[Exception] = field(default_factory=list, repr=False)
 
 
+# read/write
+
+
+def read_object(
+    path: PathLike,
+    /,
+    *,
+    decompress: bool = False,
+    dataclass_hook: _DataclassHook | None = None,
+    objects: AbstractSet[type[Any]] | None = None,
+    redirects: Mapping[str, type[Any]] | None = None,
+) -> Any:
+    """Read an object from disk."""
+    data = read_binary(path, decompress=decompress)
+    return deserialize(
+        data, dataclass_hook=dataclass_hook, objects=objects, redirects=redirects
+    )
+
+
+def write_object(
+    obj: Any,
+    path: PathLike,
+    /,
+    *,
+    before: Callable[[Any], Any] | None = None,
+    globalns: StrMapping | None = None,
+    localns: StrMapping | None = None,
+    warn_name_errors: bool = False,
+    dataclass_hook: _DataclassHook | None = None,
+    dataclass_defaults: bool = False,
+    compress: bool = False,
+    overwrite: bool = False,
+) -> None:
+    """Write an object to disk."""
+    data = serialize(
+        obj,
+        before=before,
+        globalns=globalns,
+        localns=localns,
+        warn_name_errors=warn_name_errors,
+        dataclass_hook=dataclass_hook,
+        dataclass_defaults=dataclass_defaults,
+    )
+    write_formatted_json(data, path, compress=compress, overwrite=overwrite)
+
+
 __all__ = [
     "DeserializeError",
     "GetLogRecordsOutput",
@@ -1172,5 +1252,7 @@ __all__ = [
     "SerializeError",
     "deserialize",
     "get_log_records",
+    "read_object",
     "serialize",
+    "write_object",
 ]
