@@ -2,8 +2,10 @@ from __future__ import annotations
 
 from zoneinfo import ZoneInfo
 
+HongKong = ZoneInfo("Asia/Hong_Kong")
+Tokyo = ZoneInfo("Asia/Tokyo")
 USCentral = ZoneInfo("US/Central")
 USEastern = ZoneInfo("US/Eastern")
 
 
-__all__ = ["USCentral", "USEastern"]
+__all__ = ["HongKong", "Tokyo", "USCentral", "USEastern"]
